# == Schema Information
#
# Table name: benchmark_configurations
#
#  id                  :integer          not null, primary key
#  name                :string
#  language_id         :integer
#  runtime_id          :integer
#  operating_system_id :integer
#  created_at          :datetime
#  updated_at          :datetime
#

class BenchmarkConfiguration < ActiveRecord::Base

  has_many :benchmark_batches, dependent: :destroy, inverse_of: :benchmark_configuration
  belongs_to :language, inverse_of: :benchmark_configurations
  belongs_to :runtime, inverse_of: :benchmark_configurations
  belongs_to :operating_system, inverse_of: :benchmark_configurations

  def name
    "#{runtime.name} (#{language.name}) on #{operating_system.name}"
  end

  def all_benchmark_infos
    result = Set.new
    benchmark_batches.each do |batch|
      batch.benchmark_results.each { |r| result.add r.benchmark_info }
    end
    result
  end

  def all_dates_when_performed
    SortedSet.new(benchmark_batches.map(&:performed_at).flatten).to_a
  end

  def results_at(time)
    batch = benchmark_batches.where(performed_at: time).first
    if batch
      all_benchmark_infos.map do |info|
        batch.result_for(info)
      end
    else
      []
    end
  end

  def results_of(benchmark_info, time)
    batch_with_given_date = benchmark_batches
    .where(performed_at: time)
    .first

    if batch_with_given_date
      batch_with_given_date
      .benchmark_results(benchmark_info: benchmark_info)
      .first
    else
      nil
    end
  end

end
