# == Schema Information
#
# Table name: runtimes
#
#  id          :integer          not null, primary key
#  name        :string(255)
#  language_id :integer
#  created_at  :datetime
#  updated_at  :datetime
#

class Runtime < ActiveRecord::Base

  belongs_to :language, inverse_of: :runtimes
  has_many :benchmark_configurations, inverse_of: :runtime

  validates_presence_of :language_id, :name

  def all_benchmark_infos
    result = Set.new
    benchmark_batches.each do |batch|
      batch.benchmark_results.each { |r| result.add r.benchmark_info }
    end
    result
  end

  def all_dates_when_performed
    SortedSet.new(benchmark_batches.map(&:performed_at).flatten).to_a
  end

  def results_at(time)
    batch = benchmark_batches.where(performed_at: time).first
    all_benchmark_infos.map do |info|
      batch.result_for(info)
    end
  end

  def results_of(benchmark_info, time)
    batch_with_given_date = benchmark_batches
    .where(performed_at: time)
    .first

    if batch_with_given_date
      batch_with_given_date
      .benchmark_results(benchmark_info: benchmark_info)
      .first
    else
      nil
    end
  end

end
